function effect = rsdetectable (y, X, A, alfa, pwr)
% RSDETECTABLE - Calculation of detectable effects size for the one-degree general linear hypothesis H0: A*beta=d against the two-sided alternative. The right-hand side d of H0 is not needed in the calculation, hence not received as a parameter. Version 0.1.14.
%
% The syntax is
% effect = rslinhyp(y, X, A, [alfa], [pwr])
%
% Arguments
% y is an N by 1 column vector of data to fit.
% X is an N by a matrix containing the values of a predictors used in the regression, including a possible constant 1 column.
% A is a row vector of lenght a matrix with, forming the linear hypothesis to test. Note that A can only have one row here!
% [alfa] is the significance level used in the test. Defaults to 0.05 if none is entered.
% [pwr] is the desired power of the test. Defaults to 0.90.
%
% Return values
% effect is the smallest (positive) true value of a*beta-d which leads to the rejection of H0 with the given probability [pwr].
%
% References
% Searle: Linear Models, Wiley, 1971, Chapter 6: Testing of general linear hypotheses
%
% (The square root of the f-statistic is t-distributied in the one degree of freedom case;
% this is exploited to make a simple power calculation. Reference needed.)


% Copyright (C) 2016 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2016


% Check input
if nargin < 3 || nargin > 5,
    error('Try help rsdetectable for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rslinhyp for instructions.');
end
if cond(X'*X) > 1e6,
    warning('The normal equation is ill-conditioned. If the matrix X does not have linearly dependent columns, try standardizing the factor data.');
end
if size(A,1) > 1,
    error('A has more than one row. This is not yet implemented.');
end
if nargin == 3,
    alfa=0.05
    pwr=0.90
end
if nargin == 4,
    pwr=0.90
end
% Still check that SSE>0 so that testing is possible.
b=X\y;
pred=X*b;
r=y-pred;
SSE=r'*r;
if SSE < 1e-6,
    warning('Your model seems to fit the data perfectly, then no hypothesis testing is possible. If the equation X*b=y is in fact overdetermined, try normalizing the factor data.');
end

% Perform power calculation
[N,a]=size(X);
dfE=N-a;
MSE=SSE/dfE;
standarderror=sqrt(MSE*A*inv(X'*X)*A');
effect=(tinv(1-alfa/2,dfE)-tinv(1-pwr,dfE))*standarderror;       % Scale back to natural scale

end
